@@ -112,8 +112,8 @@ class Contract:
     request_cls = None
 
     def __init__(self, method, *args):
-        self.testcase_pre = _create_testcase(method, '@%s pre-hook' % self.name)
-        self.testcase_post = _create_testcase(method, '@%s post-hook' % self.name)
+        self.testcase_pre = _create_testcase(method, f'@{self.name} pre-hook')
+        self.testcase_post = _create_testcase(method, f'@{self.name} post-hook')
         self.args = args
 
     def add_pre_hook(self, request, results):
@@ -172,8 +172,8 @@ def _create_testcase(method, desc):
 
     class ContractTestCase(TestCase):
         def __str__(_self):
-            return "[%s] %s (%s)" % (spider, method.__name__, desc)
+            return f"[{spider}] {method.__name__} ({desc})"
 
-    name = '%s_%s' % (spider, method.__name__)
+    name = f'{spider}_{method.__name__}'
     setattr(ContractTestCase, name, lambda x: x)
     return ContractTestCase(name)
