@@ -10,7 +10,7 @@ from scrapy.http.headers import Headers
 from scrapy.http.request import Request
 from scrapy.http.request.form import FormRequest
 from scrapy.http.request.rpc import XmlRpcRequest
-from scrapy.http.request.json_request import JSONRequest
+from scrapy.http.request.json_request import JsonRequest
 
 from scrapy.http.response import Response
 from scrapy.http.response.html import HtmlResponse
