@@ -1,6 +1,6 @@
 def obsolete_setter(setter, attrname):
     def newsetter(self, value):
         c = self.__class__.__name__
-        msg = "%s.%s is not modifiable, use %s.replace() instead" % (c, attrname, c)
+        msg = f"{c}.{attrname} is not modifiable, use {c}.replace() instead"
         raise AttributeError(msg)
     return newsetter
