@@ -4,7 +4,7 @@
 class TestExtension:
 
     def __init__(self, settings):
-        settings.set('TEST1', "%s + %s" % (settings['TEST1'], 'started'))
+        settings.set('TEST1', f"{settings['TEST1']} + started")
 
     @classmethod
     def from_crawler(cls, crawler):
