@@ -1,6 +1,5 @@
 import json
 import logging
-import sys
 from ipaddress import IPv4Address
 from socket import gethostbyname
 from urllib.parse import urlparse
@@ -20,6 +19,9 @@ from scrapy.http.response import Response
 from scrapy.utils.python import to_unicode
 from tests.mockserver import MockServer
 from tests.spiders import (
+    AsyncDefAsyncioGenComplexSpider,
+    AsyncDefAsyncioGenLoopSpider,
+    AsyncDefAsyncioGenSpider,
     AsyncDefAsyncioReqsReturnSpider,
     AsyncDefAsyncioReturnSingleElementSpider,
     AsyncDefAsyncioReturnSpider,
@@ -79,7 +81,7 @@ class CrawlTestCase(TestCase):
         total_time = times[-1] - times[0]
         average = total_time / (len(times) - 1)
         self.assertTrue(average > delay * tolerance,
-                        "download delay too small: %s" % average)
+                        f"download delay too small: {average}")
 
         # Ensure that the same test parameters would cause a failure if no
         # download delay is set. Otherwise, it means we are using a combination
@@ -204,7 +206,7 @@ with multiples lines
 '''})
         crawler = self.runner.create_crawler(SimpleSpider)
         with LogCapture() as log:
-            yield crawler.crawl(self.mockserver.url("/raw?{0}".format(query)), mockserver=self.mockserver)
+            yield crawler.crawl(self.mockserver.url(f"/raw?{query}"), mockserver=self.mockserver)
         self.assertEqual(str(log).count("Got response 200"), 1)
 
     @defer.inlineCallbacks
@@ -333,6 +335,19 @@ class CrawlSpiderTestCase(TestCase):
     def tearDown(self):
         self.mockserver.__exit__(None, None, None)
 
+    @defer.inlineCallbacks
+    def _run_spider(self, spider_cls):
+        items = []
+
+        def _on_item_scraped(item):
+            items.append(item)
+
+        crawler = self.runner.create_crawler(spider_cls)
+        crawler.signals.connect(_on_item_scraped, signals.item_scraped)
+        with LogCapture() as log:
+            yield crawler.crawl(self.mockserver.url("/status?n=200"), mockserver=self.mockserver)
+        return log, items, crawler.stats
+
     @defer.inlineCallbacks
     def test_crawlspider_with_parse(self):
         self.runner.crawl(CrawlSpiderWithParseMethod, mockserver=self.mockserver)
@@ -377,15 +392,7 @@ class CrawlSpiderTestCase(TestCase):
     @mark.only_asyncio()
     @defer.inlineCallbacks
     def test_async_def_asyncio_parse_items_list(self):
-        items = []
-
-        def _on_item_scraped(item):
-            items.append(item)
-
-        crawler = self.runner.create_crawler(AsyncDefAsyncioReturnSpider)
-        crawler.signals.connect(_on_item_scraped, signals.item_scraped)
-        with LogCapture() as log:
-            yield crawler.crawl(self.mockserver.url("/status?n=200"), mockserver=self.mockserver)
+        log, items, _ = yield self._run_spider(AsyncDefAsyncioReturnSpider)
         self.assertIn("Got response 200", str(log))
         self.assertIn({'id': 1}, items)
         self.assertIn({'id': 2}, items)
@@ -405,52 +412,29 @@ class CrawlSpiderTestCase(TestCase):
         self.assertIn("Got response 200", str(log))
         self.assertIn({"foo": 42}, items)
 
-    @mark.skipif(sys.version_info < (3, 6), reason="Async generators require Python 3.6 or higher")
     @mark.only_asyncio()
     @defer.inlineCallbacks
     def test_async_def_asyncgen_parse(self):
-        from tests.py36._test_crawl import AsyncDefAsyncioGenSpider
-        crawler = self.runner.create_crawler(AsyncDefAsyncioGenSpider)
-        with LogCapture() as log:
-            yield crawler.crawl(self.mockserver.url("/status?n=200"), mockserver=self.mockserver)
+        log, _, stats = yield self._run_spider(AsyncDefAsyncioGenSpider)
         self.assertIn("Got response 200", str(log))
-        itemcount = crawler.stats.get_value('item_scraped_count')
+        itemcount = stats.get_value('item_scraped_count')
         self.assertEqual(itemcount, 1)
 
-    @mark.skipif(sys.version_info < (3, 6), reason="Async generators require Python 3.6 or higher")
     @mark.only_asyncio()
     @defer.inlineCallbacks
     def test_async_def_asyncgen_parse_loop(self):
-        items = []
-
-        def _on_item_scraped(item):
-            items.append(item)
-
-        from tests.py36._test_crawl import AsyncDefAsyncioGenLoopSpider
-        crawler = self.runner.create_crawler(AsyncDefAsyncioGenLoopSpider)
-        crawler.signals.connect(_on_item_scraped, signals.item_scraped)
-        with LogCapture() as log:
-            yield crawler.crawl(self.mockserver.url("/status?n=200"), mockserver=self.mockserver)
+        log, items, stats = yield self._run_spider(AsyncDefAsyncioGenLoopSpider)
         self.assertIn("Got response 200", str(log))
-        itemcount = crawler.stats.get_value('item_scraped_count')
+        itemcount = stats.get_value('item_scraped_count')
         self.assertEqual(itemcount, 10)
         for i in range(10):
             self.assertIn({'foo': i}, items)
 
-    @mark.skipif(sys.version_info < (3, 6), reason="Async generators require Python 3.6 or higher")
     @mark.only_asyncio()
     @defer.inlineCallbacks
     def test_async_def_asyncgen_parse_complex(self):
-        items = []
-
-        def _on_item_scraped(item):
-            items.append(item)
-
-        from tests.py36._test_crawl import AsyncDefAsyncioGenComplexSpider
-        crawler = self.runner.create_crawler(AsyncDefAsyncioGenComplexSpider)
-        crawler.signals.connect(_on_item_scraped, signals.item_scraped)
-        yield crawler.crawl(mockserver=self.mockserver)
-        itemcount = crawler.stats.get_value('item_scraped_count')
+        _, items, stats = yield self._run_spider(AsyncDefAsyncioGenComplexSpider)
+        itemcount = stats.get_value('item_scraped_count')
         self.assertEqual(itemcount, 156)
         # some random items
         for i in [1, 4, 21, 22, 207, 311]:
@@ -461,11 +445,9 @@ class CrawlSpiderTestCase(TestCase):
     @mark.only_asyncio()
     @defer.inlineCallbacks
     def test_async_def_asyncio_parse_reqs_list(self):
-        crawler = self.runner.create_crawler(AsyncDefAsyncioReqsReturnSpider)
-        with LogCapture() as log:
-            yield crawler.crawl(self.mockserver.url("/status?n=200"), mockserver=self.mockserver)
+        log, *_ = yield self._run_spider(AsyncDefAsyncioReqsReturnSpider)
         for req_id in range(3):
-            self.assertIn("Got response 200, req_id %d" % req_id, str(log))
+            self.assertIn(f"Got response 200, req_id {req_id}", str(log))
 
     @defer.inlineCallbacks
     def test_response_ssl_certificate_none(self):
