@@ -17,7 +17,7 @@ class ResponseTypesTest(unittest.TestCase):
         ]
         for source, cls in mappings:
             retcls = responsetypes.from_filename(source)
-            assert retcls is cls, "%s ==> %s != %s" % (source, retcls, cls)
+            assert retcls is cls, f"{source} ==> {retcls} != {cls}"
 
     def test_from_content_disposition(self):
         mappings = [
@@ -32,7 +32,7 @@ class ResponseTypesTest(unittest.TestCase):
         ]
         for source, cls in mappings:
             retcls = responsetypes.from_content_disposition(source)
-            assert retcls is cls, "%s ==> %s != %s" % (source, retcls, cls)
+            assert retcls is cls, f"{source} ==> {retcls} != {cls}"
 
     def test_from_content_type(self):
         mappings = [
@@ -47,7 +47,7 @@ class ResponseTypesTest(unittest.TestCase):
         ]
         for source, cls in mappings:
             retcls = responsetypes.from_content_type(source)
-            assert retcls is cls, "%s ==> %s != %s" % (source, retcls, cls)
+            assert retcls is cls, f"{source} ==> {retcls} != {cls}"
 
     def test_from_body(self):
         mappings = [
@@ -58,7 +58,7 @@ class ResponseTypesTest(unittest.TestCase):
         ]
         for source, cls in mappings:
             retcls = responsetypes.from_body(source)
-            assert retcls is cls, "%s ==> %s != %s" % (source, retcls, cls)
+            assert retcls is cls, f"{source} ==> {retcls} != {cls}"
 
     def test_from_headers(self):
         mappings = [
@@ -70,7 +70,7 @@ class ResponseTypesTest(unittest.TestCase):
         for source, cls in mappings:
             source = Headers(source)
             retcls = responsetypes.from_headers(source)
-            assert retcls is cls, "%s ==> %s != %s" % (source, retcls, cls)
+            assert retcls is cls, f"{source} ==> {retcls} != {cls}"
 
     def test_from_args(self):
         # TODO: add more tests that check precedence between the different arguments
@@ -86,7 +86,7 @@ class ResponseTypesTest(unittest.TestCase):
         ]
         for source, cls in mappings:
             retcls = responsetypes.from_args(**source)
-            assert retcls is cls, "%s ==> %s != %s" % (source, retcls, cls)
+            assert retcls is cls, f"{source} ==> {retcls} != {cls}"
 
     def test_custom_mime_types_loaded(self):
         # check that mime.types files shipped with scrapy are loaded
