@@ -1,24 +1,23 @@
 import unittest
+
+from scrapy import Spider
 from scrapy.http import Request
-from scrapy.item import BaseItem
+from scrapy.item import Item
 from scrapy.utils.spider import iterate_spider_output, iter_spider_classes
 
-from scrapy.spiders import CrawlSpider
 
-
-class MyBaseSpider(CrawlSpider):
-    pass # abstract spider
-
-class MySpider1(MyBaseSpider):
+class MySpider1(Spider):
     name = 'myspider1'
 
-class MySpider2(MyBaseSpider):
+
+class MySpider2(Spider):
     name = 'myspider2'
 
+
 class UtilsSpidersTestCase(unittest.TestCase):
 
     def test_iterate_spider_output(self):
-        i = BaseItem()
+        i = Item()
         r = Request('http://scrapytest.org')
         o = object()
 
@@ -32,6 +31,6 @@ class UtilsSpidersTestCase(unittest.TestCase):
         it = iter_spider_classes(tests.test_utils_spider)
         self.assertEqual(set(it), {MySpider1, MySpider2})
 
+
 if __name__ == "__main__":
     unittest.main()
-
