@@ -213,7 +213,7 @@ def create_guess_scheme_t(args):
     def do_expected(self):
         url = guess_scheme(args[0])
         assert url.startswith(args[1]), \
-            'Wrong scheme guessed: for `%s` got `%s`, expected `%s...`' % (args[0], url, args[1])
+            f'Wrong scheme guessed: for `{args[0]}` got `{url}`, expected `{args[1]}...`'
     return do_expected
 
 
@@ -254,7 +254,7 @@ for k, args in enumerate(
     start=1,
 ):
     t_method = create_guess_scheme_t(args)
-    t_method.__name__ = 'test_uri_%03d' % k
+    t_method.__name__ = f'test_uri_{k:03}'
     setattr(GuessSchemeTest, t_method.__name__, t_method)
 
 # TODO: the following tests do not pass with current implementation
@@ -269,7 +269,7 @@ for k, args in enumerate(
     start=1,
 ):
     t_method = create_skipped_scheme_t(args)
-    t_method.__name__ = 'test_uri_skipped_%03d' % k
+    t_method.__name__ = f'test_uri_skipped_{k:03}'
     setattr(GuessSchemeTest, t_method.__name__, t_method)
 
 
